@@ -203,6 +203,14 @@ gda_client_init (GdaClient *client, GdaClientClass *klass)
 	client->priv->connections = NULL;
 }
 
+static void
+remove_weak_ref (gpointer key, gpointer value, gpointer user_data)
+{
+	LoadedProvider *prv = (LoadedProvider *) value;
+
+	g_object_weak_unref (G_OBJECT (prv->provider), (GWeakNotify) provider_weak_cb, G_OBJECT (user_data));
+}
+
 static void
 gda_client_finalize (GObject *object)
 {
@@ -213,6 +221,7 @@ gda_client_finalize (GObject *object)
 	/* free memory */
 	gda_client_close_all_connections (client);
 
+	g_hash_table_foreach (client->priv->providers, (GHFunc) remove_weak_ref, client);
 	g_hash_table_foreach (client->priv->providers, (GHFunc) free_hash_provider, NULL);
 	g_hash_table_destroy (client->priv->providers);
 	client->priv->providers = NULL;
@@ -249,7 +258,7 @@ gda_client_get_type (void)
 /**
  * gda_client_new
  *
- * Create a new #GdaClient object, which is the entry point for libgda
+ * Creates a new #GdaClient object, which is the entry point for libgda
  * client applications. This object, once created, can be used for
  * opening new database connections and activating other services
  * available to GDA clients.
@@ -271,12 +280,15 @@ gda_client_new (void)
  * @dsn: data source name.
  * @username: user name.
  * @password: password for @username.
- * @options: Options for the connection.
+ * @options: options for the connection (see #GdaConnectionOptions).
  *
- * Establish a connection to a data source.
+ * Establishes a connection to a data source. The connection will be opened
+ * if no identical connection is available in the #GdaClient connection pool,
+ * and re-used if available. If you dont want to share the connection,
+ * specify #GDA_CONNECTION_OPTIONS_DONT_SHARE as one of the flags in
+ * the @options parameter.
  *
- * This function is the most common way of opening database connections with
- * libgda.
+ * This function is the way of opening database connections with libgda.
  *
  * Returns: the opened connection if successful, %NULL if there is
  * an error.
@@ -302,11 +314,15 @@ gda_client_open_connection (GdaClient *client,
 	}
 
 	/* search for the connection in our private list */
-	cnc = gda_client_find_connection (client, dsn, username, password);
-	if (cnc) {
-		g_object_ref (G_OBJECT (cnc));
-		gda_client_notify_connection_opened_event (client, cnc);
-		return cnc;
+	if (! (options & GDA_CONNECTION_OPTIONS_DONT_SHARE)) {
+		cnc = gda_client_find_connection (client, dsn, username, password);
+		if (cnc &&
+		    ! (gda_connection_get_options (cnc) & GDA_CONNECTION_OPTIONS_DONT_SHARE)) {
+			g_object_ref (G_OBJECT (cnc));
+			gda_client_notify_connection_opened_event (client, cnc);
+			gda_config_free_data_source_info (dsn_info);
+			return cnc;
+		}
 	}
 
 	/* try to find provider in our hash table */
@@ -331,6 +347,7 @@ gda_client_open_connection (GdaClient *client,
 		/* load the new provider */
 		prv = g_new0 (LoadedProvider, 1);
 		prv->handle = g_module_open (prv_info->location, G_MODULE_BIND_LAZY);
+		gda_config_free_provider_info (prv_info);
 		if (!prv->handle) {
 			emit_client_error (client, NULL, g_module_error ());
 			gda_config_free_data_source_info (dsn_info);
@@ -398,14 +415,14 @@ gda_client_open_connection (GdaClient *client,
  * @client: a #GdaClient object.
  * @provider_id: provider ID to connect to.
  * @cnc_string: connection string.
+ * @options: options for the connection (see #GdaConnectionOptions).
  *
- * Open a connection given a provider ID and a connection string. This
+ * Opens a connection given a provider ID and a connection string. This
  * allows applications to open connections without having to create
- * a data source in the configuration.  The format of @cnc_string is
- * similar to postgresql and mysql connection strings.
- * 
+ * a data source in the configuration. The format of @cnc_string is
+ * similar to PostgreSQL and MySQL connection strings.
  *
- * Returns: the opened connection if successful, NULL if there is
+ * Returns: the opened connection if successful, %NULL if there is
  * an error.
  */
 GdaConnection *
@@ -417,9 +434,24 @@ gda_client_open_connection_from_string (GdaClient *client,
 	GdaDataSourceInfo *dsn_info;
 	GdaConnection *cnc;
 	static gint count = 0;
+	GList *l;
 
 	g_return_val_if_fail (GDA_IS_CLIENT (client), NULL);
 	g_return_val_if_fail (provider_id != NULL, NULL);
+	
+	if (! (options & GDA_CONNECTION_OPTIONS_DONT_SHARE)) {
+		for (l = client->priv->connections; l != NULL; l = l->next) {
+			const gchar *tmp_prov, *tmp_cnc_string;
+	
+			cnc = GDA_CONNECTION (l->data);
+			tmp_prov = gda_connection_get_provider (cnc);
+			tmp_cnc_string = gda_connection_get_cnc_string (cnc);
+	
+			if (strcmp(provider_id, tmp_prov) == 0
+			    && (cnc_string != NULL && strcmp (cnc_string, tmp_cnc_string) == 0))
+				return cnc;
+		}
+	}
 
 	/* create a temporary DSNInfo */
 	dsn_info = g_new (GdaDataSourceInfo, 1);
@@ -438,7 +470,8 @@ gda_client_open_connection_from_string (GdaClient *client,
 				     dsn_info->password);
 
 	/* open the connection */
-	cnc = gda_client_open_connection (client, dsn_info->name,
+	cnc = gda_client_open_connection (client,
+					  dsn_info->name,
 					  dsn_info->username,
 					  dsn_info->password,
 					  options);
@@ -454,7 +487,7 @@ gda_client_open_connection_from_string (GdaClient *client,
  * gda_client_get_connection_list
  * @client: a #GdaClient object.
  *
- * Get the list of all open connections in the given #GdaClient.
+ * Gets the list of all open connections in the given #GdaClient object.
  * The GList returned is an internal pointer, so DON'T TRY TO
  * FREE IT.
  *
@@ -474,14 +507,14 @@ gda_client_get_connection_list (GdaClient *client)
  * @username: user name.
  * @password: password for @username.
  *
- * Look for an open connection given a data source name (per libgda
+ * Looks for an open connection given a data source name (per libgda
  * configuration), a username and a password.
  *
  * This function iterates over the list of open connections in the
  * given #GdaClient and looks for one that matches the given data source
  * name, username and password.
  *
- * Returns: a pointer to the found connection, or NULL if it could not
+ * Returns: a pointer to the found connection, or %NULL if it could not
  * be found.
  */
 GdaConnection *
@@ -511,7 +544,7 @@ gda_client_find_connection (GdaClient *client,
 		tmp_usr = gda_connection_get_username (cnc);
 		tmp_pwd = gda_connection_get_password (cnc);
 
-		if (((!tmp_dsn && !dsn_info->cnc_string) || !strcmp (tmp_dsn, dsn_info->cnc_string)) &&
+		if (((!tmp_dsn && !dsn_info->name) || !strcmp (tmp_dsn, dsn_info->name)) &&
 		    ((!tmp_usr && !username) || !strcmp (tmp_usr, username)) &&
 		    ((!tmp_pwd && !password) || !strcmp (tmp_pwd, password))) {
 			gda_config_free_data_source_info (dsn_info);
@@ -528,15 +561,22 @@ gda_client_find_connection (GdaClient *client,
  * gda_client_close_all_connections
  * @client: a #GdaClient object.
  *
- * Close all connections opened by the given #GdaClient object.
+ * Closes all connections opened by the given #GdaClient object.
  */
 void
 gda_client_close_all_connections (GdaClient *client)
 {
 	g_return_if_fail (GDA_IS_CLIENT (client));
 
-	g_list_foreach (client->priv->connections, (GFunc) g_object_unref, NULL);
-	g_list_free (client->priv->connections);
+	while (client->priv->connections != NULL) {
+		GdaConnection *cnc = client->priv->connections->data;
+
+		g_assert (GDA_IS_CONNECTION (cnc));
+
+		client->priv->connections = g_list_remove (client->priv->connections, cnc);
+		g_object_unref (cnc);
+	}
+
 	client->priv->connections = NULL;
 }
 
@@ -547,7 +587,7 @@ gda_client_close_all_connections (GdaClient *client)
  * @event: event ID.
  * @params: parameters associated with the event.
  *
- * Notify an event to the given #GdaClient's listeners. The event can be
+ * Notifies an event to the given #GdaClient's listeners. The event can be
  * anything (see #GdaClientEvent) ranging from a connection opening
  * operation, to changes made to a table in an underlying database.
  */
@@ -558,8 +598,11 @@ gda_client_notify_event (GdaClient *client,
 			 GdaParameterList *params)
 {
 	g_return_if_fail (GDA_IS_CLIENT (client));
-	g_signal_emit (G_OBJECT (client), gda_client_signals[EVENT_NOTIFICATION], 0,
-		       cnc, event, params);
+
+	if (g_list_find (client->priv->connections, cnc)) {
+		g_signal_emit (G_OBJECT (client), gda_client_signals[EVENT_NOTIFICATION], 0,
+			       cnc, event, params);
+	}
 }
 
 /**
@@ -568,7 +611,7 @@ gda_client_notify_event (GdaClient *client,
  * @cnc: a #GdaConnection object.
  * @error: the error to be notified.
  *
- * Notify the given #GdaClient of the #GDA_CLIENT_EVENT_ERROR event.
+ * Notifies the given #GdaClient of the #GDA_CLIENT_EVENT_ERROR event.
  */
 void
 gda_client_notify_error_event (GdaClient *client, GdaConnection *cnc, GdaError *error)
@@ -591,7 +634,8 @@ gda_client_notify_error_event (GdaClient *client, GdaConnection *cnc, GdaError *
  * @client: a #GdaClient object.
  * @cnc: a #GdaConnection object.
  *
- * Notify the given #GdaClient of the #GDA_CLIENT_EVENT_CONNECTION_OPENED event.
+ * Notifies the given #GdaClient of the #GDA_CLIENT_EVENT_CONNECTION_OPENED 
+ * event.
  */
 void
 gda_client_notify_connection_opened_event (GdaClient *client, GdaConnection *cnc)
@@ -607,7 +651,8 @@ gda_client_notify_connection_opened_event (GdaClient *client, GdaConnection *cnc
  * @client: a #GdaClient object.
  * @cnc: a #GdaConnection object.
  *
- * Notify the given #GdaClient of the #GDA_CLIENT_EVENT_CONNECTION_CLOSED event.
+ * Notifies the given #GdaClient of the #GDA_CLIENT_EVENT_CONNECTION_CLOSED 
+ * event.
  */
 void
 gda_client_notify_connection_closed_event (GdaClient *client, GdaConnection *cnc)
@@ -624,7 +669,8 @@ gda_client_notify_connection_closed_event (GdaClient *client, GdaConnection *cnc
  * @cnc: a #GdaConnection object.
  * @xaction: a #GdaTransaction object.
  *
- * Notify the given #GdaClient of the #GDA_CLIENT_EVENT_TRANSACTION_STARTED event.
+ * Notifies the given #GdaClient of the #GDA_CLIENT_EVENT_TRANSACTION_STARTED
+ * event.
  */
 void
 gda_client_notify_transaction_started_event (GdaClient *client, GdaConnection *cnc, GdaTransaction *xaction)
@@ -648,7 +694,8 @@ gda_client_notify_transaction_started_event (GdaClient *client, GdaConnection *c
  * @cnc: a #GdaConnection object.
  * @xaction: a #GdaTransaction object.
  *
- * Notify the given #GdaClient of the #GDA_CLIENT_EVENT_TRANSACTION_COMMITTED event.
+ * Notifies the given #GdaClient of the #GDA_CLIENT_EVENT_TRANSACTION_COMMITTED
+ * event.
  */
 void
 gda_client_notify_transaction_committed_event (GdaClient *client, GdaConnection *cnc, GdaTransaction *xaction)
@@ -672,7 +719,8 @@ gda_client_notify_transaction_committed_event (GdaClient *client, GdaConnection
  * @cnc: a #GdaConnection object.
  * @xaction: a #GdaTransaction object.
  *
- * Notify the given #GdaClient of the #GDA_CLIENT_EVENT_TRANSACTION_CANCELLED event.
+ * Notifies the given #GdaClient of the #GDA_CLIENT_EVENT_TRANSACTION_CANCELLED 
+ * event.
  */
 void
 gda_client_notify_transaction_cancelled_event (GdaClient *client, GdaConnection *cnc, GdaTransaction *xaction)
@@ -695,7 +743,7 @@ gda_client_notify_transaction_cancelled_event (GdaClient *client, GdaConnection
  * @client: a #GdaClient object.
  * @xaction: a #GdaTransaction object.
  *
- * Start a transaction on all connections being managed by the given
+ * Starts a transaction on all connections being managed by the given
  * #GdaClient. It is important to note that this operates on all
  * connections opened within a #GdaClient, which could not be what
  * you're looking for.
@@ -704,8 +752,8 @@ gda_client_notify_transaction_cancelled_event (GdaClient *client, GdaConnection
  * #gda_connection_begin_transaction, #gda_connection_commit_transaction
  * and #gda_connection_rollback_transaction.
  *
- * Returns: TRUE if all transactions could be started successfully,
- * or FALSE if one of them fails.
+ * Returns: %TRUE if all transactions could be started successfully,
+ * or %FALSE if one of them fails.
  */
 gboolean
 gda_client_begin_transaction (GdaClient *client, GdaTransaction *xaction)
@@ -730,7 +778,7 @@ gda_client_begin_transaction (GdaClient *client, GdaTransaction *xaction)
  * @client: a #GdaClient object.
  * @xaction: a #GdaTransaction object.
  *
- * Commit a running transaction on all connections being managed by the given
+ * Commits a running transaction on all connections being managed by the given
  * #GdaClient. It is important to note that this operates on all
  * connections opened within a #GdaClient, which could not be what
  * you're looking for.
@@ -739,8 +787,8 @@ gda_client_begin_transaction (GdaClient *client, GdaTransaction *xaction)
  * #gda_connection_begin_transaction, #gda_connection_commit_transaction
  * and #gda_connection_rollback_transaction.
  *
- * Returns: TRUE if all transactions could be committed successfully,
- * or FALSE if one of them fails.
+ * Returns: %TRUE if all transactions could be committed successfully,
+ * or %FALSE if one of them fails.
  */
 gboolean
 gda_client_commit_transaction (GdaClient *client, GdaTransaction *xaction)
@@ -774,8 +822,8 @@ gda_client_commit_transaction (GdaClient *client, GdaTransaction *xaction)
  * #gda_connection_begin_transaction, #gda_connection_commit_transaction
  * and #gda_connection_rollback_transaction.
  *
- * Returns: TRUE if all transactions could be cancelled successfully,
- * or FALSE if one of them fails.
+ * Returns: %TRUE if all transactions could be cancelled successfully,
+ * or %FALSE if one of them fails.
  */
 gboolean
 gda_client_rollback_transaction (GdaClient *client, GdaTransaction *xaction)
