@@ -23,7 +23,6 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <config.h>
 #include <glib.h>
 #include <gmodule.h>
 #include <libgda/gda-config.h>
@@ -38,14 +37,14 @@
 #define LIBGDA_USER_CONFIG_FILE LIBGDA_USER_CONFIG_DIR G_DIR_SEPARATOR_S \
 				"config"
 
-//FIXME: many functions are not thread safe!
+/* FIXME: many functions are not thread safe! */
 
 typedef struct {
     gchar *name;
     gchar *type;
     gchar *value;
-    //gchar *mtime;
-    //gchar *muser;
+    /* gchar *mtime; */
+    /* gchar *muser; */
 } gda_config_entry;
 
 typedef struct {
@@ -218,10 +217,11 @@ gda_config_parse_config_file (gchar *buffer, gint len)
 static gda_config_client *
 get_config_client ()
 {
-	//FIXME: if we're updating or writing config_client,
-	//	 wait until the operation finishes
+	/* FIXME: if we're updating or writing config_client,
+	 *	  wait until the operation finishes
+	 */
 	if (config_client == NULL){
-		gint len;
+		gsize  len;
 		gchar *full_file;
 		gchar *user_config;
 
@@ -460,12 +460,12 @@ write_config_file ()
 
 /**
  * gda_config_get_string
- * @path: path to the configuration entry
+ * @path: path to the configuration entry.
  *
  * Gets the value of the specified configuration entry as a string. You
- * are then responsible to free the returned string
+ * are then responsible to free the returned string.
  *
- * Returns: the value stored at the given entry
+ * Returns: the value stored at the given entry.
  */
 gchar *
 gda_config_get_string (const gchar *path)
@@ -487,11 +487,11 @@ gda_config_get_string (const gchar *path)
 
 /**
  * gda_config_get_int
- * @path: path to the configuration entry
+ * @path: path to the configuration entry.
  *
- * Gets the value of the specified configuration entry as an integer
+ * Gets the value of the specified configuration entry as an integer.
  *
- * Returns: the value stored at the given entry
+ * Returns: the value stored at the given entry.
  */
 gint
 gda_config_get_int (const gchar *path)
@@ -513,11 +513,11 @@ gda_config_get_int (const gchar *path)
 
 /**
  * gda_config_get_float
- * @path: path to the configuration entry
+ * @path: path to the configuration entry.
  *
- * Gets the value of the specified configuration entry as a float
+ * Gets the value of the specified configuration entry as a float.
  *
- * Returns: the value stored at the given entry
+ * Returns: the value stored at the given entry.
  */
 gdouble
 gda_config_get_float (const gchar *path)
@@ -539,11 +539,11 @@ gda_config_get_float (const gchar *path)
 
 /**
  * gda_config_get_boolean
- * @path: path to the configuration entry
+ * @path: path to the configuration entry.
  *
- * Gets the value of the specified configuration entry as a boolean
+ * Gets the value of the specified configuration entry as a boolean.
  *
- * Returns: the value stored at the given entry
+ * Returns: the value stored at the given entry.
  */
 gboolean
 gda_config_get_boolean (const gchar *path)
@@ -565,10 +565,10 @@ gda_config_get_boolean (const gchar *path)
 
 /**
  * gda_config_set_string
- * @path: path to the configuration entry
- * @new_value: new value
+ * @path: path to the configuration entry.
+ * @new_value: new value.
  *
- * Sets the given configuration entry to contain a string
+ * Sets the given configuration entry to contain a string.
  */
 void
 gda_config_set_string (const gchar *path, const gchar *new_value)
@@ -598,7 +598,9 @@ gda_config_set_string (const gchar *path, const gchar *new_value)
 		g_free (section_path);
 	} else {
 		g_free (entry->value);
+		g_free (entry->type);
 		entry->value = g_strdup (new_value);
+		entry->type = g_strdup ("string");
 	}
 
 	write_config_file ();
@@ -607,10 +609,10 @@ gda_config_set_string (const gchar *path, const gchar *new_value)
 
 /**
  * gda_config_set_int
- * @path: path to the configuration entry
- * @new_value: new value
+ * @path: path to the configuration entry.
+ * @new_value: new value.
  *
- * Sets the given configuration entry to contain an integer
+ * Sets the given configuration entry to contain an integer.
  */
 void
 gda_config_set_int (const gchar *path, gint new_value)
@@ -642,7 +644,9 @@ gda_config_set_int (const gchar *path, gint new_value)
 		g_free (section_path);
 	} else {
 		g_free (entry->value);
+		g_free (entry->type);
 		entry->value = g_strdup_printf ("%d", new_value);
+		entry->type = g_strdup ("long");
 	}
 
 	write_config_file ();
@@ -651,10 +655,10 @@ gda_config_set_int (const gchar *path, gint new_value)
 
 /**
  * gda_config_set_float
- * @path: path to the configuration entry
- * @new_value: new value
+ * @path: path to the configuration entry.
+ * @new_value: new value.
  *
- * Sets the given configuration entry to contain a float
+ * Sets the given configuration entry to contain a float.
  */
 void
 gda_config_set_float (const gchar * path, gdouble new_value)
@@ -686,7 +690,9 @@ gda_config_set_float (const gchar * path, gdouble new_value)
 		g_free (section_path);
 	} else {
 		g_free (entry->value);
+		g_free (entry->type);
 		entry->value = g_strdup_printf ("%f", new_value);
+		entry->type = g_strdup ("float");
 	}
 
 	write_config_file ();
@@ -695,10 +701,10 @@ gda_config_set_float (const gchar * path, gdouble new_value)
 
 /**
  * gda_config_set_boolean
- * @path: path to the configuration entry
- * @new_value: new value
+ * @path: path to the configuration entry.
+ * @new_value: new value.
  *
- * Sets the given configuration entry to contain a boolean
+ * Sets the given configuration entry to contain a boolean.
  */
 void
 gda_config_set_boolean (const gchar *path, gboolean new_value)
@@ -730,7 +736,9 @@ gda_config_set_boolean (const gchar *path, gboolean new_value)
 		g_free (section_path);
 	} else {
 		g_free (entry->value);
+		g_free (entry->type);
 		entry->value = g_strdup_printf ("%d", new_value);
+		entry->type = g_strdup ("bool");
 	}
 
 	write_config_file ();
@@ -739,9 +747,9 @@ gda_config_set_boolean (const gchar *path, gboolean new_value)
 
 /**
  * gda_config_remove_section
- * @path: path to the configuration section
+ * @path: path to the configuration section.
  *
- * Remove the given section from the configuration database
+ * Removes the given section from the configuration database.
  */
 void
 gda_config_remove_section (const gchar *path)
@@ -766,9 +774,9 @@ gda_config_remove_section (const gchar *path)
 
 /**
  * gda_config_remove_key
- * @path: path to the configuration entry
+ * @path: path to the configuration entry.
  *
- * Remove the given entry from the configuration database
+ * Removes the given entry from the configuration database.
  * If the section is empty, also remove the section.
  */
 void
@@ -825,12 +833,12 @@ gda_config_remove_key (const gchar *path)
 
 /**
  * gda_config_has_section
- * @path: path to the configuration section
+ * @path: path to the configuration section.
  *
  * Checks whether the given section exists in the configuration
- * system
+ * system.
  *
- * Returns: TRUE if the section exists, FALSE otherwise
+ * Returns: %TRUE if the section exists, %FALSE otherwise.
  */
 gboolean
 gda_config_has_section (const gchar *path)
@@ -850,11 +858,11 @@ gda_config_has_section (const gchar *path)
 
 /**
  * gda_config_has_key
- * @path: path to the configuration key
+ * @path: path to the configuration key.
  *
- * Check whether the given key exists in the configuration system
+ * Checks whether the given key exists in the configuration system.
  *
- * Returns: TRUE if the entry exists, FALSE otherwise
+ * Returns: %TRUE if the entry exists, %FALSE otherwise.
  */
 gboolean
 gda_config_has_key (const gchar *path)
@@ -876,12 +884,12 @@ gda_config_has_key (const gchar *path)
 
 /**
  * gda_config_get_type
- * @path: path to the configuration key
+ * @path: path to the configuration key.
  *
  * Gets a string representing the type of the value of the given key.
  * The caller is responsible of freeing the returned value.
  *
- * Returns: NULL if not found. Otherwise: "string", "float", "long", "bool".
+ * Returns: %NULL if not found. Otherwise: "string", "float", "long", "bool".
  */
 gchar *
 gda_config_get_type (const gchar *path)
@@ -906,14 +914,14 @@ gda_config_get_type (const gchar *path)
 
 /**
  * gda_config_list_sections
- * @path: path for root dir
+ * @path: path for root dir.
  *
- * Return a GList containing the names of all the sections available
+ * Returns a GList containing the names of all the sections available
  * under the given root directory.
  *
- * To free the returned value, you can use #gda_config_free_list
+ * To free the returned value, you can use #gda_config_free_list.
  *
- * Returns: a list containing all the section names
+ * Returns: a list containing all the section names.
  */
 GList *
 gda_config_list_sections (const gchar *path)
@@ -956,13 +964,13 @@ gda_config_list_sections (const gchar *path)
 
 /**
  * gda_config_list_keys
- * @path: path for root dir
+ * @path: path for root dir.
  *
  * Returns a list of all keys that exist under the given path.
  *
- * To free the returned value, you can use #gda_config_free_list
+ * To free the returned value, you can use #gda_config_free_list.
  *
- * Returns: a list containing all the key names
+ * Returns: a list containing all the key names.
  */
 GList *
 gda_config_list_keys (const gchar * path)
@@ -1013,10 +1021,10 @@ gda_config_list_keys (const gchar * path)
 
 /**
  * gda_config_free_list
- * @list: list to be freed
+ * @list: list to be freed.
  *
- * Free all memory used by the given GList, which must be the return value
- * from either #gda_config_list_sections and #gda_config_list_keys
+ * Frees all memory used by the given GList, which must be the return value
+ * from either #gda_config_list_sections and #gda_config_list_keys.
  */
 void
 gda_config_free_list (GList *list)
@@ -1028,7 +1036,7 @@ gda_config_free_list (GList *list)
 /**
  * gda_config_get_provider_list
  *
- * Return a list of all providers currently installed in
+ * Returns a list of all providers currently installed in
  * the system. Each of the nodes in the returned GList
  * is a #GdaProviderInfo. To free the returned list,
  * call the #gda_config_free_provider_list function.
@@ -1070,6 +1078,7 @@ gda_config_get_provider_list (void)
 				     name, NULL);
 		handle = g_module_open (path, G_MODULE_BIND_LAZY);
 		if (!handle) {
+			g_warning (_("Error: %s"), g_module_error ());
 			g_free (path);
 			continue;
 		}
@@ -1114,7 +1123,7 @@ gda_config_get_provider_list (void)
  * gda_config_free_provider_list
  * @list: the list to be freed.
  *
- * Free a list of #GdaProviderInfo structures.
+ * Frees a list of #GdaProviderInfo structures.
  */
 void
 gda_config_free_provider_list (GList *list)
@@ -1133,6 +1142,11 @@ gda_config_free_provider_list (GList *list)
 
 /**
  * gda_config_get_provider_by_name
+ * @name: name of the provider to search for.
+ *
+ * Gets a #GdaProviderInfo structure from the provider list given its name.
+ *
+ * Returns: a #GdaProviderInfo structure, if found, or %NULL if not found.
  */
 GdaProviderInfo *
 gda_config_get_provider_by_name (const gchar *name)
@@ -1158,8 +1172,46 @@ gda_config_get_provider_by_name (const gchar *name)
 	return NULL;
 }
 
+/**
+ * gda_provider_info_copy
+ * @src: provider information to get a copy from.
+ *
+ * Creates a new #GdaProviderInfo structure from an existing one.
+
+ * Returns: a newly allocated #GdaProviderInfo with contains a copy of 
+ * information in @src.
+ */
+GdaProviderInfo*
+gda_provider_info_copy (GdaProviderInfo *src)
+{
+	GdaProviderInfo *info;
+	GList *list = NULL;
+	GList *list_src = NULL;
+
+	g_return_val_if_fail (src != NULL, NULL);
+
+	info = g_new0 (GdaProviderInfo, 1);
+	info->id = g_strdup (src->id);
+	info->location = g_strdup (src->location);
+	info->description = g_strdup (src->description);
+
+	/* Deep copy: */
+	list_src = src->gda_params;
+	while (list_src) {
+		list = g_list_append (list, g_strdup (list_src->data));
+		list_src = g_list_next (list_src);
+	}
+
+	info->gda_params = list;
+
+	return info;
+}
+
 /**
  * gda_config_free_provider_info
+ * @provider_info: provider information to free.
+ *
+ * Deallocates all memory associated to the given #GdaProviderInfo.
  */
 void
 gda_config_free_provider_info (GdaProviderInfo *provider_info)
@@ -1177,6 +1229,13 @@ gda_config_free_provider_info (GdaProviderInfo *provider_info)
 
 /**
  * gda_config_get_data_source_list
+ *
+ * Returns a list of all data sources currently configured in the system.
+ * Each of the nodes in the returned GList is a #GdaDataSourceInfo.
+ * To free the returned list, call the #gda_config_free_data_source_list
+ * function.
+ *
+ * Returns: a GList of #GdaDataSourceInfo structures.
  */
 GList *
 gda_config_get_data_source_list (void)
@@ -1228,6 +1287,12 @@ gda_config_get_data_source_list (void)
 
 /**
  * gda_config_find_data_source
+ * @name: name of the data source to search for.
+ *
+ * Gets a #GdaDataSourceInfo structure from the data source list given its 
+ * name.
+ *
+ * Returns: a #GdaDataSourceInfo structure, if found, or %NULL if not found.
  */
 GdaDataSourceInfo *
 gda_config_find_data_source (const gchar *name)
@@ -1255,6 +1320,12 @@ gda_config_find_data_source (const gchar *name)
 
 /**
  * gda_config_copy_data_source_info
+ * @src: data source information to get a copy from.
+ *
+ * Creates a new #GdaDataSourceInfo structure from an existing one.
+
+ * Returns: a newly allocated #GdaDataSourceInfo with contains a copy of 
+ * information in @src.
  */
 GdaDataSourceInfo *
 gda_config_copy_data_source_info (GdaDataSourceInfo *src)
@@ -1276,6 +1347,9 @@ gda_config_copy_data_source_info (GdaDataSourceInfo *src)
 
 /**
  * gda_config_free_data_source_info
+ * @info: data source information to free.
+ *
+ * Deallocates all memory associated to the given #GdaDataSourceInfo.
  */
 void
 gda_config_free_data_source_info (GdaDataSourceInfo *info)
@@ -1294,6 +1368,9 @@ gda_config_free_data_source_info (GdaDataSourceInfo *info)
 
 /**
  * gda_config_free_data_source_list
+ * @list: the list to be freed.
+ *
+ * Frees a list of #GdaDataSourceInfo structures.
  */
 void
 gda_config_free_data_source_list (GList *list)
@@ -1310,6 +1387,15 @@ gda_config_free_data_source_list (GList *list)
 
 /**
  * gda_config_get_data_source_model
+ *
+ * Fills and returns a new #GdaDataModel object using information from all 
+ * data sources which are currently configured in the system.
+ *
+ * Rows are separated in 6 columns: 
+ * 'Name', 'Provider', 'Connection string', 'Description', 'Username' and 
+ * 'Password'.
+ *
+ * Returns: a new #GdaDataModel object. 
  */
 GdaDataModel *
 gda_config_get_data_source_model (void)
@@ -1352,12 +1438,12 @@ gda_config_get_data_source_model (void)
 
 /**
  * gda_config_save_data_source
- * @name: Name for the data source to be saved.
- * @provider: Provider ID for the new data source.
- * @cnc_string: Connection string for the new data source.
- * @description: Description for the new data source.
- * @username: User name for the new data source.
- * @password: Password to use when authenticating @username.
+ * @name: name for the data source to be saved.
+ * @provider: provider ID for the new data source.
+ * @cnc_string: connection string for the new data source.
+ * @description: description for the new data source.
+ * @username: user name for the new data source.
+ * @password: password to use when authenticating @username.
  *
  * Adds a new data source (or update an existing one) to the GDA
  * configuration, based on the parameters given.
@@ -1415,9 +1501,30 @@ gda_config_save_data_source (const gchar *name,
 	g_string_free (str, TRUE);
 }
 
+/**
+ * gda_config_save_data_source_info
+ * @dsn_info: a #GdaDataSourceInfo structure.
+ *
+ * Saves a data source in the libgda configuration given a
+ * #GdaDataSourceInfo structure containing all the information
+ * about the data source.
+ */
+void
+gda_config_save_data_source_info (GdaDataSourceInfo *dsn_info)
+{
+	g_return_if_fail (dsn_info != NULL);
+
+	gda_config_save_data_source (dsn_info->name,
+				     dsn_info->provider,
+				     dsn_info->cnc_string,
+				     dsn_info->description,
+				     dsn_info->username,
+				     dsn_info->password);
+}
+
 /**
  * gda_config_remove_data_source
- * @name: Name for the data source to be removed.
+ * @name: name for the data source to be removed.
  *
  * Removes the given data source from the GDA configuration.
  */
@@ -1506,6 +1613,10 @@ gda_config_add_listener (const gchar *path,
 
 /**
  * gda_config_remove_listener
+ * @id: the ID of the listener to remove.
+ *
+ * Removes a configuration listener previously installed with
+ * #gda_config_add_listener, given its ID.
  */
 void
 gda_config_remove_listener (guint id)
@@ -1524,3 +1635,32 @@ gda_config_remove_listener (guint id)
 	}
 }
 
+GType
+gda_provider_info_get_type (void)
+{
+	static GType our_type = 0;
+
+	if (our_type == 0)
+		our_type = g_boxed_type_register_static ("GdaProviderInfo",
+			(GBoxedCopyFunc) gda_provider_info_copy,
+			(GBoxedFreeFunc) gda_config_free_provider_info);
+
+  return our_type;
+}
+
+GType
+gda_data_source_info_get_type (void)
+{
+	static GType our_type = 0;
+
+	if (our_type == 0)
+		our_type = g_boxed_type_register_static ("GdaDataSourceInfo",
+			(GBoxedCopyFunc) gda_config_copy_data_source_info,
+			(GBoxedFreeFunc) gda_config_free_data_source_info);
+
+  return our_type;
+}
+
+
+
+
