@@ -27,6 +27,7 @@
 #include <libgda/gda-connection.h>
 #include <libgda/gda-data-model.h>
 #include <libgda/gda-quark-list.h>
+#include <libgda/gda-transaction.h>
 
 G_BEGIN_DECLS
 
@@ -51,6 +52,8 @@ struct _GdaServerProviderClass {
 	void (* last_connection_gone) (GdaServerProvider *provider);
 
 	/* virtual methods */
+	const gchar * (* get_version) (GdaServerProvider *provider);
+
 	gboolean (* open_connection) (GdaServerProvider *provider,
 				      GdaConnection *cnc,
 				      GdaQuarkList *params,
@@ -59,6 +62,20 @@ struct _GdaServerProviderClass {
 	gboolean (* close_connection) (GdaServerProvider *provider,
 				       GdaConnection *cnc);
 
+	const gchar * (* get_server_version) (GdaServerProvider *provider,
+					      GdaConnection *cnc);
+	const gchar * (* get_database) (GdaServerProvider *provider,
+					GdaConnection *cnc);
+	gboolean (* change_database) (GdaServerProvider *provider,
+				      GdaConnection *cnc,
+				      const gchar *name);
+	gboolean (* create_database) (GdaServerProvider *provider,
+				      GdaConnection *cnc,
+				      const gchar *name);
+	gboolean (* drop_database) (GdaServerProvider *provider,
+				    GdaConnection *cnc,
+				    const gchar *name);
+
 	GList * (* execute_command) (GdaServerProvider *provider,
 				     GdaConnection *cnc,
 				     GdaCommand *cmd,
@@ -66,13 +83,13 @@ struct _GdaServerProviderClass {
 
 	gboolean (* begin_transaction) (GdaServerProvider *provider,
 					GdaConnection *cnc,
-					const gchar *trans_id);
+					GdaTransaction *xaction);
 	gboolean (* commit_transaction) (GdaServerProvider *provider,
 					 GdaConnection *cnc,
-					 const gchar *trans_id);
+					 GdaTransaction *xaction);
 	gboolean (* rollback_transaction) (GdaServerProvider *provider,
 					   GdaConnection *cnc,
-					   const gchar *trans_id);
+					   GdaTransaction *xaction);
 
 	gboolean (* supports) (GdaServerProvider *provider,
 			       GdaConnection *cnc,
@@ -82,9 +99,14 @@ struct _GdaServerProviderClass {
 				       GdaConnection *cnc,
 				       GdaConnectionSchema schema,
 				       GdaParameterList *params);
+
+	gboolean (* create_blob) (GdaServerProvider *provider,
+				  GdaConnection *cnc,
+				  GdaBlob *blob);
 };
 
 GType    gda_server_provider_get_type (void);
+const gchar *gda_server_provider_get_version (GdaServerProvider *provider);
 gboolean gda_server_provider_open_connection (GdaServerProvider *provider,
 					      GdaConnection *cnc,
 					      GdaQuarkList *params,
@@ -93,6 +115,20 @@ gboolean gda_server_provider_open_connection (GdaServerProvider *provider,
 gboolean gda_server_provider_close_connection (GdaServerProvider *provider,
 					       GdaConnection *cnc);
 
+const gchar *gda_server_provider_get_server_version (GdaServerProvider *provider,
+						     GdaConnection *cnc);
+const gchar *gda_server_provider_get_database (GdaServerProvider *provider,
+					       GdaConnection *cnc);
+gboolean gda_server_provider_change_database (GdaServerProvider *provider,
+					      GdaConnection *cnc,
+					      const gchar *name);
+gboolean gda_server_provider_create_database (GdaServerProvider *provider,
+					      GdaConnection *cnc,
+					      const gchar *name);
+gboolean gda_server_provider_drop_database (GdaServerProvider *provider,
+					    GdaConnection *cnc,
+					    const gchar *name);
+
 GList   *gda_server_provider_execute_command (GdaServerProvider *provider,
 					      GdaConnection *cnc,
 					      GdaCommand *cmd,
@@ -100,13 +136,13 @@ GList   *gda_server_provider_execute_command (GdaServerProvider *provider,
 
 gboolean gda_server_provider_begin_transaction (GdaServerProvider *provider,
 						GdaConnection *cnc,
-						const gchar *trans_id);
+						GdaTransaction *xaction);
 gboolean gda_server_provider_commit_transaction (GdaServerProvider *provider,
 						 GdaConnection *cnc,
-						 const gchar *trans_id);
+						 GdaTransaction *xaction);
 gboolean gda_server_provider_rollback_transaction (GdaServerProvider *provider,
 						   GdaConnection *cnc,
-						   const gchar *trans_id);
+						   GdaTransaction *xaction);
 
 gboolean gda_server_provider_supports (GdaServerProvider *provider,
 				       GdaConnection *cnc,
@@ -117,6 +153,10 @@ GdaDataModel *gda_server_provider_get_schema (GdaServerProvider *provider,
 					      GdaConnectionSchema schema,
 					      GdaParameterList *params);
 
+gboolean gda_server_provider_create_blob (GdaServerProvider *provider,
+					  GdaConnection *cnc,
+					  GdaBlob *blob);
+
 G_END_DECLS
 
 #endif
