@@ -5,6 +5,7 @@
  *	Michael Lausch <michael@lausch.at>
  *	Rodrigo Moya <rodrigo@gnome-db.org>
  *	Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
+ *	Juan-Mariano de Goyeneche <jmseyas@dit.upm.es> (BLOB issues)
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -35,6 +36,18 @@
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 
+GType
+gda_value_get_gtype (void)
+{
+	static GType our_type = 0;
+
+	if (our_type == 0)
+		our_type = g_boxed_type_register_static ("GdaValue",
+			(GBoxedCopyFunc) gda_value_copy,
+			(GBoxedFreeFunc) gda_value_free);
+	return our_type;
+}
+
 /*
  * Private functions
  */
@@ -59,6 +72,18 @@ clear_value (GdaValue *value)
 		g_free (value->value.v_string);
 		value->value.v_string = NULL;
 		break;
+	case GDA_VALUE_TYPE_BLOB :
+		gda_blob_free_data (&value->value.v_blob);
+		memset (&value->value.v_blob, 0, sizeof (GdaBlob));
+		break;
+	case GDA_VALUE_TYPE_BINARY :
+		g_free (value->value.v_binary);
+		value->value.v_binary = NULL;
+		break;
+	case GDA_VALUE_TYPE_NUMERIC :
+		g_free (value->value.v_numeric.number);
+		value->value.v_numeric.number = NULL;
+		break;
 	default :
 		break;
 	}
@@ -279,9 +304,9 @@ set_from_string (GdaValue *value, const gchar *as_string)
 /**
  * gda_value_new_null
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_NULL.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_NULL.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_null (void)
@@ -298,9 +323,9 @@ gda_value_new_null (void)
  * gda_value_new_bigint
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_BIGINT with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_BIGINT with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_bigint (gint64 val)
@@ -317,9 +342,9 @@ gda_value_new_bigint (gint64 val)
  * gda_value_new_biguint
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_BIGUINT with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_BIGUINT with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 
 GdaValue *gda_value_new_biguint(guint64 val) {
@@ -333,10 +358,11 @@ GdaValue *gda_value_new_biguint(guint64 val) {
 /**
  * gda_value_new_binary
  * @val: value to set for the new #GdaValue.
+ * @size: the size of the memory pool pointer to by @val.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_BINARY with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_BINARY with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_binary (gconstpointer val, glong size)
@@ -353,9 +379,9 @@ gda_value_new_binary (gconstpointer val, glong size)
  * gda_value_new_boolean
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_BOOLEAN with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_BOOLEAN with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_boolean (gboolean val)
@@ -372,9 +398,9 @@ gda_value_new_boolean (gboolean val)
  * gda_value_new_date
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_DATE with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_DATE with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_date (const GdaDate *val)
@@ -391,9 +417,9 @@ gda_value_new_date (const GdaDate *val)
  * gda_value_new_double
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_DOUBLE with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_DOUBLE with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_double (gdouble val)
@@ -410,10 +436,10 @@ gda_value_new_double (gdouble val)
  * gda_value_new_geometric_point
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_GEOMETRIC_POINT with value
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_GEOMETRIC_POINT with value
  * @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_geometric_point (const GdaGeometricPoint *val)
@@ -430,7 +456,7 @@ gda_value_new_geometric_point (const GdaGeometricPoint *val)
  * gda_value_new_gobject
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_GOBJECT with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_GOBJECT with value @val.
  *
  * Returns: the newly created #GdaValue.
  */
@@ -449,9 +475,9 @@ gda_value_new_gobject (const GObject *val)
  * gda_value_new_integer
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_INTEGER with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_INTEGER with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_integer (gint val)
@@ -468,9 +494,9 @@ gda_value_new_integer (gint val)
  * gda_value_new_uinteger
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_UINTEGER with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_UINTEGER with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 
 GdaValue *gda_value_new_uinteger(guint val) {
@@ -486,9 +512,9 @@ GdaValue *gda_value_new_uinteger(guint val) {
  * gda_value_new_list
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_LIST with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_LIST with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_list (const GdaValueList *val)
@@ -505,9 +531,9 @@ gda_value_new_list (const GdaValueList *val)
  + gda_value_new_money
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_MONEY with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_MONEY with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_money (const GdaMoney *val)
@@ -524,9 +550,9 @@ gda_value_new_money (const GdaMoney *val)
  * gda_value_new_numeric
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_NUMERIC with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_NUMERIC with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_numeric (const GdaNumeric *val)
@@ -543,9 +569,9 @@ gda_value_new_numeric (const GdaNumeric *val)
  * gda_value_new_single
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_SINGLE with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_SINGLE with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_single (gfloat val)
@@ -562,9 +588,9 @@ gda_value_new_single (gfloat val)
  * gda_value_new_smallint
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_SMALLINT with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_SMALLINT with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_smallint (gshort val)
@@ -581,9 +607,9 @@ gda_value_new_smallint (gshort val)
  * gda_value_new_smalluint
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_SMALLUINT with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_SMALLUINT with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_smalluint (gushort val)
@@ -600,9 +626,9 @@ gda_value_new_smalluint (gushort val)
  * gda_value_new_string
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_STRING with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_STRING with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_string (const gchar *val)
@@ -619,9 +645,9 @@ gda_value_new_string (const gchar *val)
  * gda_value_new_time
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_TIME with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_TIME with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_time (const GdaTime *val)
@@ -638,9 +664,9 @@ gda_value_new_time (const GdaTime *val)
  * gda_value_new_timestamp
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_TIMESTAMP with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_TIMESTAMP with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_timestamp (const GdaTimestamp *val)
@@ -657,8 +683,8 @@ gda_value_new_timestamp (const GdaTimestamp *val)
  * gda_value_new_timestamp_from_timet
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_TIMESTAMP with value @val (of type
- * time_t).
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_TIMESTAMP with value @val 
+ * (of type time_t).
  *
  * Returns: the newly created #GdaValue.
  */
@@ -690,9 +716,9 @@ gda_value_new_timestamp_from_timet (time_t val)
  * gda_value_new_tinyint
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_TINYINT with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_TINYINT with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_tinyint (gchar val)
@@ -709,9 +735,9 @@ gda_value_new_tinyint (gchar val)
  * gda_value_new_tinyuint
  * @val: value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_TINYUINT with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_TINYUINT with value @val.
  *
- * Returns: The newly created #GdaValue.
+ * Returns: the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_tinyuint (guchar val)
@@ -728,7 +754,7 @@ gda_value_new_tinyuint (guchar val)
  * gda_value_new_type
  * @val: Value to set for the new #GdaValue.
  *
- * Make a new #GdaValue of type #GDA_VALUE_TYPE_TYPE with value @val.
+ * Makes a new #GdaValue of type #GDA_VALUE_TYPE_TYPE with value @val.
  *
  * Returns: the newly created #GdaValue.
  */
@@ -747,11 +773,11 @@ gda_value_new_type (GdaValueType val)
 /**
  * gda_value_new_from_string
  * @as_string: stringified representation of the value.
- * @type: tha new value type.
+ * @type: the new value type.
  *
- * Make a new #GdaValue of type @type from its string representation.
+ * Makes a new #GdaValue of type @type from its string representation.
  *
- * Returns: The newly created #GdaValue or NULL if the string representation
+ * Returns: the newly created #GdaValue or %NULL if the string representation
  * cannot be converted to the specified @type.
  */
 GdaValue *
@@ -773,11 +799,11 @@ gda_value_new_from_string (const gchar *as_string, GdaValueType type)
  * gda_value_new_from_xml
  * @xml: a XML node representing the value.
  *
- * Create a GdaValue from a XML representation of it. That XML
+ * Creates a GdaValue from a XML representation of it. That XML
  * node corresponds to the following string representation:
  *    <value type="gdatype">value</value>
  *
- * Returns:  The newly created #GdaValue.
+ * Returns:  the newly created #GdaValue.
  */
 GdaValue *
 gda_value_new_from_xml (const xmlNodePtr node)
@@ -793,7 +819,7 @@ gda_value_new_from_xml (const xmlNodePtr node)
 	value = g_new0 (GdaValue, 1);
 	if (!gda_value_set_from_string (value,
 					xmlNodeGetContent (node),
-					gda_type_from_string (xmlGetProp (node, "type")))) {
+					gda_type_from_string (xmlGetProp (node, "gdatype")))) {
 		g_free (value);
 		value = NULL;
 	}
@@ -805,7 +831,7 @@ gda_value_new_from_xml (const xmlNodePtr node)
  * gda_value_free
  * @value: the resource to free.
  *
- * Deallocates all memory associated to  a #GdaValue.
+ * Deallocates all memory associated to a #GdaValue.
  */
 void
 gda_value_free (GdaValue *value)
@@ -820,7 +846,7 @@ gda_value_free (GdaValue *value)
  * gda_value_get_type
  * @value: value to get the type from.
  *
- * Retrieve the type of the given value.
+ * Retrieves the type of the given value.
  *
  * Returns: the #GdaValueType of the value.
  */
@@ -851,10 +877,10 @@ gda_value_is_null (const GdaValue *value)
  * gda_value_is_number
  * @value: a #GdaValue.
  *
- * Get whether the value stored in the given #GdaValue is of
+ * Gets whether the value stored in the given #GdaValue is of
  * numeric type or not.
  *
- * Returns: TRUE if a number, FALSE otherwise.
+ * Returns: %TRUE if a number, %FALSE otherwise.
  */
 gboolean
 gda_value_is_number (const GdaValue *value)
@@ -912,6 +938,10 @@ gda_value_copy (const GdaValue *value)
 		copy->binary_length = value->binary_length;
 		memcpy (copy->value.v_binary, value->value.v_binary, value->binary_length);
 		break;
+	case GDA_VALUE_TYPE_BLOB :
+		memcpy (&copy->value.v_blob, &value->value.v_blob,
+			sizeof (GdaBlob));
+		break;
 	case GDA_VALUE_TYPE_BOOLEAN :
 		copy->value.v_boolean = value->value.v_boolean;
 		break;
@@ -948,6 +978,7 @@ gda_value_copy (const GdaValue *value)
 		break;
 	case GDA_VALUE_TYPE_NUMERIC :
 		memcpy (&copy->value.v_numeric, &value->value.v_numeric, sizeof (GdaNumeric));
+		copy->value.v_numeric.number = g_strdup (value->value.v_numeric.number);
 		break;
 	case GDA_VALUE_TYPE_SINGLE :
 		copy->value.v_single = value->value.v_single;
@@ -1022,9 +1053,7 @@ gda_value_set_bigint (GdaValue *value, gint64 val)
  * gda_value_get_biguint
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 guint64
 gda_value_get_biguint (const GdaValue *value)
@@ -1056,9 +1085,7 @@ gda_value_set_biguint (GdaValue *value, guint64 val)
  * @value: a #GdaValue whose value we want to get.
  * @size: holder for length of data.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 G_CONST_RETURN gpointer
 gda_value_get_binary (const GdaValue *value, glong *size)
@@ -1095,13 +1122,48 @@ gda_value_set_binary (GdaValue *value, gconstpointer val, glong size)
 	memcpy (value->value.v_binary, val, size);
 }
 
+/**
+ * gda_value_get_blob
+ * @value: a #GdaValue whose value we want to get.
+ *
+ * Returns: the value stored in @value.
+ */
+const GdaBlob *
+gda_value_get_blob (const GdaValue *value)
+{
+	g_return_val_if_fail (value != NULL, NULL);
+	g_return_val_if_fail (gda_value_isa (value, GDA_VALUE_TYPE_BLOB), NULL);
+	return (const GdaBlob *) &value->value.v_blob;
+}
+
+/**
+ * gda_value_set_blob
+ * @value: a #GdaValue that will store @val.
+ * @val: value to be stored in @value.
+ *
+ * Stores @val into @value.
+ */
+void
+gda_value_set_blob (GdaValue *value, const GdaBlob *val)
+{
+	g_return_if_fail (value != NULL);
+	g_return_if_fail (val != NULL);
+
+	clear_value (value);
+	value->type = GDA_VALUE_TYPE_BLOB;
+	value->value.v_blob.priv_data = val->priv_data;
+	value->value.v_blob.open = val->open;
+	value->value.v_blob.read = val->read;
+	value->value.v_blob.write = val->write;
+	value->value.v_blob.lseek = val->lseek;
+	value->value.v_blob.close = val->close;
+}
+
 /**
  * gda_value_get_boolean
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 gboolean
 gda_value_get_boolean (const GdaValue *value)
@@ -1132,9 +1194,7 @@ gda_value_set_boolean (GdaValue *value, gboolean val)
  * gda_value_get_date
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 G_CONST_RETURN GdaDate *
 gda_value_get_date (const GdaValue *value)
@@ -1168,9 +1228,7 @@ gda_value_set_date (GdaValue *value, const GdaDate *val)
  * gda_value_get_double
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 gdouble
 gda_value_get_double (const GdaValue *value)
@@ -1201,9 +1259,7 @@ gda_value_set_double (GdaValue *value, gdouble val)
  * gda_value_get_geometric_point
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 G_CONST_RETURN GdaGeometricPoint *
 gda_value_get_geometric_point (const GdaValue *value)
@@ -1236,7 +1292,7 @@ gda_value_set_geometric_point (GdaValue *value, const GdaGeometricPoint *val)
  * gda_value_get_gobject
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
+ * Returns: the value stored in @value.
  */
 G_CONST_RETURN GObject *
 gda_value_get_gobject (const GdaValue *value)
@@ -1269,9 +1325,7 @@ gda_value_set_gobject (GdaValue *value, const GObject *val)
  * gda_value_get_integer
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 gint
 gda_value_get_integer (const GdaValue *value)
@@ -1302,9 +1356,7 @@ gda_value_set_integer (GdaValue *value, gint val)
  * gda_value_get_list
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 G_CONST_RETURN GdaValueList *
 gda_value_get_list (const GdaValue *value)
@@ -1334,9 +1386,7 @@ gda_value_set_list (GdaValue *value, const GdaValueList *val)
  * gda_value_get_money
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 G_CONST_RETURN GdaMoney *
 gda_value_get_money (const GdaValue *value)
@@ -1384,9 +1434,7 @@ gda_value_set_null (GdaValue *value)
  * gda_value_get_numeric
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 G_CONST_RETURN GdaNumeric *
 gda_value_get_numeric (const GdaValue *value)
@@ -1420,9 +1468,7 @@ gda_value_set_numeric (GdaValue *value, const GdaNumeric *val)
  * gda_value_get_single
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 gfloat
 gda_value_get_single (const GdaValue *value)
@@ -1453,9 +1499,7 @@ gda_value_set_single (GdaValue *value, gfloat val)
  * gda_value_get_smallint
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 gshort
 gda_value_get_smallint (const GdaValue *value)
@@ -1486,9 +1530,7 @@ gda_value_set_smallint (GdaValue *value, gshort val)
  * gda_value_get_smalluint
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 gushort
 gda_value_get_smalluint (const GdaValue *value)
@@ -1519,9 +1561,7 @@ gda_value_set_smalluint (GdaValue *value, gushort val)
  * gda_value_get_string
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 const gchar *
 gda_value_get_string (const GdaValue *value)
@@ -1552,9 +1592,7 @@ gda_value_set_string (GdaValue *value, const gchar *val)
  * gda_value_get_time
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 const GdaTime *
 gda_value_get_time (const GdaValue *value)
@@ -1589,9 +1627,7 @@ gda_value_set_time (GdaValue *value, const GdaTime *val)
  * gda_value_get_timestamp
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 const GdaTimestamp *
 gda_value_get_timestamp (const GdaValue *value)
@@ -1630,9 +1666,7 @@ gda_value_set_timestamp (GdaValue *value, const GdaTimestamp *val)
  * gda_value_get_tinyint
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 gchar
 gda_value_get_tinyint (const GdaValue *value)
@@ -1663,9 +1697,7 @@ gda_value_set_tinyint (GdaValue *value, gchar val)
  * gda_value_get_tinyuint
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 guchar
 gda_value_get_tinyuint (const GdaValue *value)
@@ -1696,9 +1728,7 @@ gda_value_set_tinyuint (GdaValue *value, guchar val)
  * gda_value_get_uinteger
  * @value: a #GdaValue whose value we want to get.
  *
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
+ * Returns: the value stored in @value.
  */
 guint
 gda_value_get_uinteger (const GdaValue *value)
@@ -1729,10 +1759,8 @@ gda_value_set_uinteger (GdaValue *value, guint val)
  * gda_value_get_vtype
  * @value: a #GdaValue whose value we want to get.
  * 
- * Gets the value stored in @value.
- * 
- * Returns: the value contained in @value.
- **/
+ * Returns: the value stored in @value.
+ */
 GdaValueType
 gda_value_get_vtype (const GdaValue *value)
 {
@@ -1767,8 +1795,8 @@ gda_value_set_vtype (GdaValue *value, GdaValueType type)
  *
  * Stores the value data from its string representation as @type.
  *
- * Returns: TRUE if the value has been properly converted to @type from
- * its string representation. FALSE otherwise.
+ * Returns: %TRUE if the value has been properly converted to @type from
+ * its string representation. %FALSE otherwise.
  */
 gboolean
 gda_value_set_from_string (GdaValue *value, 
@@ -1788,12 +1816,12 @@ gda_value_set_from_string (GdaValue *value,
  * @value: a #GdaValue.
  * @from: the value to copy from.
  *
- * Set the value of a #GdaValue from another #GdaValue. This
+ * Sets the value of a #GdaValue from another #GdaValue. This
  * is different from #gda_value_copy, which creates a new #GdaValue.
  * #gda_value_set_from_value, on the other hand, copies the contents
  * of @copy into @value, which must already be allocated.
  *
- * Returns: TRUE if successful, FALSE otherwise.
+ * Returns: %TRUE if successful, %FALSE otherwise.
  */
 gboolean
 gda_value_set_from_value (GdaValue *value, const GdaValue *from)
@@ -1806,67 +1834,70 @@ gda_value_set_from_value (GdaValue *value, const GdaValue *from)
 		gda_value_set_null (value);
 		break;
 	case GDA_VALUE_TYPE_BIGINT :
-		gda_value_set_bigint (value, gda_value_get_bigint ((GdaValue *) from));
+		gda_value_set_bigint (value, gda_value_get_bigint (from));
 		break;
         case GDA_VALUE_TYPE_BIGUINT :
-                gda_value_set_biguint (value, gda_value_get_biguint ((GdaValue*) from));
+                gda_value_set_biguint (value, gda_value_get_biguint (from));
                 break;
 	case GDA_VALUE_TYPE_BINARY :
 		gda_value_set_binary (value, from->value.v_binary, from->binary_length);
 		break;
+	case GDA_VALUE_TYPE_BLOB :
+		gda_value_set_blob (value, gda_value_get_blob (from));
+		break;
 	case GDA_VALUE_TYPE_BOOLEAN :
-		gda_value_set_boolean (value, gda_value_get_boolean ((GdaValue *) from));
+		gda_value_set_boolean (value, gda_value_get_boolean (from));
 		break;
 	case GDA_VALUE_TYPE_DATE :
-		gda_value_set_date (value, gda_value_get_date ((GdaValue *) from));
+		gda_value_set_date (value, gda_value_get_date (from));
 		break;
 	case GDA_VALUE_TYPE_DOUBLE :
-		gda_value_set_double (value, gda_value_get_double ((GdaValue *) from));
+		gda_value_set_double (value, gda_value_get_double (from));
 		break;
 	case GDA_VALUE_TYPE_GEOMETRIC_POINT :
-		gda_value_set_geometric_point (value, gda_value_get_geometric_point ((GdaValue *) from));
+		gda_value_set_geometric_point (value, gda_value_get_geometric_point (from));
 		break;
 	case GDA_VALUE_TYPE_GOBJECT :
-		gda_value_set_gobject (value, gda_value_get_gobject ((GdaValue *) from));
+		gda_value_set_gobject (value, gda_value_get_gobject (from));
 		break;
 	case GDA_VALUE_TYPE_INTEGER :
-		gda_value_set_integer (value, gda_value_get_integer ((GdaValue *) from));
+		gda_value_set_integer (value, gda_value_get_integer (from));
 		break;
         case GDA_VALUE_TYPE_UINTEGER :
-                gda_value_set_uinteger (value, gda_value_get_integer ((GdaValue *)from));
+                gda_value_set_uinteger (value, gda_value_get_integer (from));
                 break;
 	case GDA_VALUE_TYPE_LIST :
-		gda_value_set_list (value, gda_value_get_list ((GdaValue *) from));
+		gda_value_set_list (value, gda_value_get_list (from));
 		break;
 	case GDA_VALUE_TYPE_MONEY :
-		gda_value_set_money (value, gda_value_get_money ((GdaValue *) from));
+		gda_value_set_money (value, gda_value_get_money (from));
 		break;
 	case GDA_VALUE_TYPE_NUMERIC :
-		gda_value_set_numeric (value, gda_value_get_numeric ((GdaValue *) from));
+		gda_value_set_numeric (value, gda_value_get_numeric (from));
 		break;
 	case GDA_VALUE_TYPE_SINGLE :
-		gda_value_set_single (value, gda_value_get_single ((GdaValue *) from));
+		gda_value_set_single (value, gda_value_get_single (from));
 		break;
 	case GDA_VALUE_TYPE_SMALLINT :
-		gda_value_set_smallint (value,gda_value_get_smallint ((GdaValue *) from));
+		gda_value_set_smallint (value,gda_value_get_smallint (from));
 		break;
         case GDA_VALUE_TYPE_SMALLUINT :
-                gda_value_set_smalluint(value,gda_value_get_smalluint((GdaValue*)from));
+                gda_value_set_smalluint(value,gda_value_get_smalluint(from));
                 break;
 	case GDA_VALUE_TYPE_STRING :
-		gda_value_set_string (value, gda_value_get_string ((GdaValue *) from));
+		gda_value_set_string (value, gda_value_get_string (from));
 		break;
 	case GDA_VALUE_TYPE_TIME :
-		gda_value_set_time (value, gda_value_get_time ((GdaValue *) from));
+		gda_value_set_time (value, gda_value_get_time (from));
 		break;
 	case GDA_VALUE_TYPE_TIMESTAMP :
-		gda_value_set_timestamp (value, gda_value_get_timestamp ((GdaValue *) from));
+		gda_value_set_timestamp (value, gda_value_get_timestamp (from));
 		break;
 	case GDA_VALUE_TYPE_TINYINT :
-		gda_value_set_tinyint (value, gda_value_get_tinyint ((GdaValue *) from));
+		gda_value_set_tinyint (value, gda_value_get_tinyint (from));
 		break;
         case GDA_VALUE_TYPE_TINYUINT :
-                gda_value_set_tinyuint(value, gda_value_get_tinyuint((GdaValue*)from));
+                gda_value_set_tinyuint(value, gda_value_get_tinyuint(from));
                 break;
 	case GDA_VALUE_TYPE_TYPE :
 		clear_value (value);
@@ -1899,6 +1930,7 @@ gda_value_stringify (const GdaValue *value)
 	const GdaGeometricPoint *point;
 	const GdaValueList *list;
 	const GdaNumeric *numeric;
+	const GdaBlob *gdablob;
 	GList *l;
 	GString *str = NULL;
 	gchar *retval = NULL;
@@ -1995,6 +2027,10 @@ gda_value_stringify (const GdaValue *value)
 		} else
 			retval = g_strdup_printf (_("NULL GObject"));
 		break;
+	case GDA_VALUE_TYPE_BLOB:
+		gdablob = gda_value_get_blob (value);
+		retval = g_strdup_printf ("%s", gdablob->stringify ((GdaBlob *) gdablob));
+		break;
 	case GDA_VALUE_TYPE_LIST:
 		list = gda_value_get_list (value);
 		for (l = (GList *) list; l != NULL; l = l->next) {
@@ -2048,9 +2084,9 @@ gda_value_stringify (const GdaValue *value)
  * @value1: a #GdaValue to compare.
  * @value2: the other #GdaValue to be compared to @value1.
  *
- * Compares two values of the <b>same</b> type.
+ * Compares two values of the same type.
  *
- * Returns: if both values have the same time, return 0 if both contains
+ * Returns: if both values have the same time, returns 0 if both contains
  * the same value, an integer less than 0 if @value1 is less than @value2 or
  * an integer greater than 0 if @value1 is greater than @value2.
  */
@@ -2077,6 +2113,10 @@ gda_value_compare (const GdaValue *value1, const GdaValue *value2)
 	case GDA_VALUE_TYPE_BOOLEAN :
 		retval = value1->value.v_boolean - value2->value.v_boolean;
 		break;
+	case GDA_VALUE_TYPE_BLOB :
+		/* FIXME: Does compare() make sense with BLOBs? */
+		retval = 0;
+		break;
 	case GDA_VALUE_TYPE_DATE :
 		retval = memcmp (&value1->value.v_date, &value2->value.v_date,
 				  sizeof (GdaDate));
@@ -2194,10 +2234,9 @@ to_string (const GdaValue *value)
  * gda_value_to_xml
  * @value: a #GdaValue.
  *
- * Serialize the given #GdaValue to a XML node string.
+ * Serializes the given #GdaValue to a XML node string.
  *
- * Returns: the XML node. Once not
- * needed anymore, you should free it.
+ * Returns: the XML node. Once not needed anymore, you should free it.
  */
 xmlNodePtr
 gda_value_to_xml (const GdaValue *value)
